from tempfile import NamedTemporaryFile, mkdtemp
from rpy2.robjects import r
import os
import sys

def dump(what):
    what.to_csv(fout, index=False)
    return fout

def r_smart_assign(label, what):
    tmpdir = mkdtemp()
    fifoname = os.path.join(tmpdir, 'myfifo')
#    print("Fifoname", fifoname, flush=True)
    fdone = fifoname + '.done'
    if os.fork():
        # Parent
        r('%s = read.csv(pipe("cat %s"), header=TRUE)' % (label, fifoname))
    else:
        # Child
        os.mkfifo(fifoname)
        with open(fifoname, 'w') as fout:
            what.to_csv(fout, index=False)
        sys._exit()

    os.remove(fifoname)
    os.rmdir(tmpdir)



def r_feather_smart_assign(label, what):
    # NOT WORKING (see below)
    import feather
    tmpdir = mkdtemp()
    fifoname = os.path.join(tmpdir, 'myfifo')
#    print("Fifoname", fifoname, flush=True)
    fdone = fifoname + '.done'
    os.mkfifo(fifoname)
    if os.fork():
        # Parent
        r('library("feather")')
        r('%s = read_feather(\"%s\")' % (label, fifoname))
    else:
        # Child
        feather.write_dataframe(what, fifoname)
        #RRuntimeError: Error in eval(expr, envir, enclos) : 
        # Invalid: File is too small to be a well-formed file

        sys._exit()

    os.remove(fifoname)
    os.rmdir(tmpdir)


def r_feather_assign(label, what):
    import feather
    fout = NamedTemporaryFile('w')
    feather.write_dataframe(what, fout.name)
    r('library("feather")')
    r('%s = read_feather(\"%s\")' % (label, fout.name))
    fout.close()


def r_assign(label, what):
    fout = NamedTemporaryFile('w')
    what.to_csv(fout, index=False)
    fout.flush()
    r('%s = read.csv(\"%s\", header=TRUE)' % (label, fout.name))
    fout.close()
    
